"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProbotOctokitCore = exports.ProbotOctokit = exports.Application = exports.Context = exports.Logger = exports.createProbot = exports.Probot = void 0;
// tslint:disable-next-line: no-var-requires
require("dotenv").config();
// TODO: remove in v11
if ("DISABLE_STATS" in process.env) {
    // tslint:disable:no-console
    console.warn('[probot] "DISABLE_STATS" is no longer used since v10');
}
if ("IGNORED_ACCOUNTS" in process.env) {
    // tslint:disable:no-console
    console.warn('[probot] "IGNORED_ACCOUNTS" is no longer used since v10');
}
const webhooks_1 = require("@octokit/webhooks");
const bunyan_1 = __importDefault(require("bunyan"));
exports.Logger = bunyan_1.default;
const lru_cache_1 = __importDefault(require("lru-cache"));
const deprecation_1 = require("deprecation");
const application_1 = require("./application");
Object.defineProperty(exports, "Application", { enumerable: true, get: function () { return application_1.Application; } });
const setup_1 = require("./apps/setup");
const context_1 = require("./context");
Object.defineProperty(exports, "Context", { enumerable: true, get: function () { return context_1.Context; } });
const octokit_1 = require("./github/octokit");
Object.defineProperty(exports, "ProbotOctokit", { enumerable: true, get: function () { return octokit_1.ProbotOctokit; } });
Object.defineProperty(exports, "ProbotOctokitCore", { enumerable: true, get: function () { return octokit_1.ProbotOctokitCore; } });
const logger_1 = require("./logger");
const log_request_errors_1 = require("./middleware/log-request-errors");
const private_key_1 = require("./private-key");
const resolver_1 = require("./resolver");
const server_1 = require("./server");
const webhook_proxy_1 = require("./webhook-proxy");
const error_handler_1 = require("./error-handler");
const webhook_transform_1 = require("./webhook-transform");
const wrap_logger_1 = require("./wrap-logger");
const get_throttle_options_1 = require("./get-throttle-options");
const get_probot_octokit_with_defaults_1 = require("./get-probot-octokit-with-defaults");
// tslint:disable:no-var-requires
const defaultAppFns = [
    require("./apps/default"),
    require("./apps/sentry"),
];
// tslint:enable:no-var-requires
class Probot {
    constructor(options) {
        if (process.env.GHE_HOST && /^https?:\/\//.test(process.env.GHE_HOST)) {
            throw new Error("Your `GHE_HOST` environment variable should not begin with https:// or http://");
        }
        if (options.cert) {
            console.warn(new deprecation_1.Deprecation(`[probot] "cert" option is deprecated. Use "privateKey" instead`));
            options.privateKey = options.cert;
        }
        //
        // Probot class-specific options (Express server & Webhooks)
        //
        options.webhookPath = options.webhookPath || "/";
        options.secret = options.secret || "development";
        // TODO: deprecate probot.logger in favor of probot.log.
        //       Use `this.log = wrapLogger(logger)` like in application.ts
        this.logger = logger_1.logger;
        this.apps = [];
        // TODO: Why is this public?
        this.options = options;
        // TODO: support redis backend for access token cache if `options.redisConfig || process.env.REDIS_URL`
        const cache = new lru_cache_1.default({
            // cache max. 15000 tokens, that will use less than 10mb memory
            max: 15000,
            // Cache for 1 minute less than GitHub expiry
            maxAge: Number(process.env.INSTALLATION_TOKEN_TTL) || 1000 * 60 * 59,
        });
        const Octokit = get_probot_octokit_with_defaults_1.getProbotOctokitWithDefaults({
            githubToken: options.githubToken,
            Octokit: options.Octokit || octokit_1.ProbotOctokit,
            appId: options.id,
            privateKey: options.privateKey,
            cache,
        });
        const octokit = new Octokit();
        const log = wrap_logger_1.wrapLogger(logger_1.logger);
        this.throttleOptions = get_throttle_options_1.getThrottleOptions({
            log,
            redisConfig: options.redisConfig,
        });
        this.state = {
            cache,
            githubToken: options.githubToken,
            log,
            Octokit,
            octokit,
            throttleOptions: this.throttleOptions,
        };
        this.webhooks = new webhooks_1.Webhooks({
            path: options.webhookPath,
            secret: options.secret,
            transform: webhook_transform_1.webhookTransform.bind(null, this.state),
        });
        // TODO: normalize error handler with code in application.ts
        this.webhooks.on("error", error_handler_1.errorHandler);
        this.server = server_1.createServer({
            webhook: this.webhooks.middleware,
            logger: logger_1.logger,
        });
    }
    static async run(appFn) {
        const pkgConf = require("pkg-conf");
        const program = require("commander");
        const readOptions = () => {
            if (Array.isArray(appFn)) {
                program
                    .usage("[options] <apps...>")
                    .option("-p, --port <n>", "Port to start the server on", process.env.PORT || 3000)
                    .option("-W, --webhook-proxy <url>", "URL of the webhook proxy service.`", process.env.WEBHOOK_PROXY_URL)
                    .option("-w, --webhook-path <path>", "URL path which receives webhooks. Ex: `/webhook`", process.env.WEBHOOK_PATH)
                    .option("-a, --app <id>", "ID of the GitHub App", process.env.APP_ID)
                    .option("-s, --secret <secret>", "Webhook secret of the GitHub App", process.env.WEBHOOK_SECRET)
                    .option("-P, --private-key <file>", "Path to certificate of the GitHub App", process.env.PRIVATE_KEY_PATH)
                    .parse(appFn);
                return {
                    privateKey: private_key_1.findPrivateKey(program.privateKey) || undefined,
                    id: program.app,
                    port: program.port,
                    secret: program.secret,
                    webhookPath: program.webhookPath,
                    webhookProxy: program.webhookProxy,
                };
            }
            const privateKey = private_key_1.findPrivateKey();
            return {
                privateKey: (privateKey && privateKey.toString()) || undefined,
                id: Number(process.env.APP_ID),
                port: Number(process.env.PORT) || 3000,
                secret: process.env.WEBHOOK_SECRET,
                webhookPath: process.env.WEBHOOK_PATH,
                webhookProxy: process.env.WEBHOOK_PROXY_URL,
            };
        };
        const options = readOptions();
        const probot = new Probot(options);
        if (!options.id || !options.privateKey) {
            if (process.env.NODE_ENV === "production") {
                if (!options.id) {
                    throw new Error("Application ID is missing, and is required to run in production mode. " +
                        "To resolve, ensure the APP_ID environment variable is set.");
                }
                else if (!options.privateKey) {
                    throw new Error("Certificate is missing, and is required to run in production mode. " +
                        "To resolve, ensure either the PRIVATE_KEY or PRIVATE_KEY_PATH environment variable is set and contains a valid certificate");
                }
            }
            probot.load(setup_1.setupApp);
        }
        else if (Array.isArray(appFn)) {
            const pkg = await pkgConf("probot");
            probot.setup(program.args.concat(pkg.apps || pkg.plugins || []));
        }
        else {
            probot.load(appFn);
        }
        probot.start();
        return probot;
    }
    /**
     * @deprecated `probot.webhook` is deprecated. Use `probot.webhooks` instead
     */
    get webhook() {
        console.warn(new deprecation_1.Deprecation(`[probot] "probot.webhook" is deprecated. Use "probot.webhooks" instead instead`));
        return this.webhooks;
    }
    receive(event) {
        this.logger.debug({ event }, "Webhook received");
        return Promise.all(this.apps.map((app) => app.receive(event)));
    }
    load(appFn) {
        if (typeof appFn === "string") {
            appFn = resolver_1.resolve(appFn);
        }
        const app = new application_1.Application({
            cache: this.state.cache,
            githubToken: this.state.githubToken,
            Octokit: this.state.Octokit,
            octokit: this.state.octokit,
            throttleOptions: this.throttleOptions,
            webhooks: this.webhooks,
        });
        // Connect the router from the app to the server
        this.server.use(app.router);
        // Initialize the ApplicationFunction
        app.load(appFn);
        this.apps.push(app);
        return app;
    }
    setup(appFns) {
        // Log all unhandled rejections
        process.on("unhandledRejection", error_handler_1.errorHandler);
        // Load the given appFns along with the default ones
        appFns.concat(defaultAppFns).forEach((appFn) => this.load(appFn));
        // Register error handler as the last middleware
        this.server.use(log_request_errors_1.logRequestErrors);
    }
    start() {
        this.httpServer = this.server
            .listen(this.options.port, () => {
            if (this.options.webhookProxy) {
                webhook_proxy_1.createWebhookProxy({
                    logger: logger_1.logger,
                    path: this.options.webhookPath,
                    port: this.options.port,
                    url: this.options.webhookProxy,
                });
            }
            logger_1.logger.info("Listening on http://localhost:" + this.options.port);
        })
            .on("error", (err) => {
            if (err.code === "EADDRINUSE") {
                logger_1.logger.error(`Port ${this.options.port} is already in use. You can define the PORT environment variable to use a different port.`);
            }
            else {
                logger_1.logger.error(err);
            }
            process.exit(1);
        });
        return this.httpServer;
    }
    stop() {
        if (!this.httpServer)
            return;
        this.httpServer.close();
    }
}
exports.Probot = Probot;
exports.createProbot = (options) => {
    console.warn(new deprecation_1.Deprecation(`[probot] "createProbot(options)" is deprecated, use "new Probot(options)" instead`));
    return new Probot(options);
};
//# sourceMappingURL=index.js.map